#R code for meta-analysis####

#'The response of bacterial and fungal alpha-diversity to heavy metals in soil, sediment, and rhizosphere: A meta-analysis'
#Authors of the R script: Signorini M. & Midolo G. 

#set the directory and put the data file into it
source('Rfunctions.meta-analysis.R') # load the functions 
load.pkgs() # run 2 times in case some packages were not installed, N.B. some packages loaded by such function are not necessary for all functions

#import data------------------------------------------------------------------------------------------------
dat<-read.csv(paste(input.pth,"cleaned_dat_ONLY_POS_SIMPS.csv",sep="/"),sep=";")
names(dat)<-c("study",names(dat)[2:31])

#summarize cases with missing variables and prints how many observations contain missing variables:
missing_data<-check_NA_values(dat)
head(missing_data)

#prepare dataset, scale and transform continuous variables, decide if include missing values or not:
set_regression_data
d<-set_regression_data(data = dat,
                       remove.na = TRUE) # if TRUE, remove all rows with any NA; if FALSE, it keeps NA

#now the final dataset is d. 


#Filter only BACTERIA entries -------------------------------------------------------------------------------
attach(d)
table(biota)
d_BAC<- d %>% filter(biota=='bac')


#null model BACTERIA Cd-Cu ----------------------------------------------------------------------------------------------
d_BAC_Cu <- d_BAC %>% filter(metal=='Cu') #to elaborate
d_BAC_Cd <- d_BAC %>% filter(metal=='Cd') #to elaborate

#null model - Cd _________________________________________________________________________________________________________________
rand = list(~ 1 | study/experiment/id, ~ time_hours_days | experiment)
m_BAC_Cd<-metafor::rma.mv(yi,get_V(d_BAC_Cd), data=d_BAC_Cd, random = rand, struct = "CAR")
m_BAC_Cd 
# estimate      se    zval    pval    ci.lb   ci.ub 
#  0.0089  0.0243  0.3657  0.7146  -0.0388  0.0566   
get_perc_change(0.0089)
get_perc_change(0.0089-0.0243)
get_perc_change(0.0089+0.0243)
#means that... 
# estimate null model BAC-Cd is --> -0.0509 --> 0.89% increase
# lower bound is --> -0.0745 --> -1.53% decrease 
# upper bound is --> -0.0273 --> 3.38% decrease
#plot bacteria null model 
d_BAC_Cd_CI<-d_BAC_Cd
d_BAC_Cd_CI$yi_lowerCI <- d_BAC_Cd_CI$yi-(1.96*sqrt(d_BAC_Cd_CI$vi))
d_BAC_Cd_CI$yi_upperCI <- d_BAC_Cd_CI$yi+(1.96*sqrt(d_BAC_Cd_CI$vi))
d_BAC_Cd_CI$yiCIsign <- d_BAC_Cd_CI$yi_upperCI*d_BAC_Cd_CI$yi_lowerCI
d_BAC_Cd_CI$LnRR <- ifelse(d_BAC_Cd_CI$yiCIsign<0, "p>=0.05", "p<0.05")
d_BAC_Cd_CI_signif<-d_BAC_Cd_CI%>%filter(LnRR == "p<0.05") #filter only significant values
d_BAC_Cd_CI$LnRR<-"p>=0.05" 
ggplot(d_BAC_Cd_CI, aes(x=yi))+
  geom_histogram(data=d_BAC_Cd_CI, binwidth = 0.05, fill="gray")+
  geom_histogram(data=d_BAC_Cd_CI_signif, binwidth = 0.05, fill="violet")+
  theme_bw()+ ylim(0,100)+
  geom_vline(xintercept = 0.0089, color = "blue", size=0.75)+
  geom_vline(xintercept = 0.0089-0.0243, color = "blue", size=0.5, linetype = "3313")+
  geom_vline(xintercept = 0.0089+0.0243, color = "blue", size=0.5, linetype = "3313")+ 
  xlab("lnRR microbial alpha-diversity")+
  theme(axis.text = element_text(color = "black", size = 12)) +
  theme(axis.title.x = element_text(color = "black", size = 12))+
  theme(axis.title.y = element_text(color = "black", size = 12)) 

#null model - Cu _____________________________________________________________________________________________________________
rand = list(~ 1 | study/experiment/id, ~ time_hours_days | experiment)
m_BAC_Cu<-metafor::rma.mv(yi,get_V(d_BAC_Cu), data=d_BAC_Cu, random = rand, struct = "CAR")
m_BAC_Cu 
# estimate      se     zval    pval    ci.lb    ci.ub 
#  -0.1032  0.0340  -3.0374  0.0024  -0.1698  -0.0366  ** 
get_perc_change(-0.1032)
get_perc_change(-0.1032-0.0340)
get_perc_change(-0.1032+0.0340)
#means that...
# estimate null model BAC-Cu is --> -0.1032 --> -9.81 % decrease
# lower bound is --> -0.1372--> -6.69 % decrease 
# upper bound is --> -0.0692 --> -12.82 % decrease
#plot bacteria null model 
d_BAC_Cu_CI<-d_BAC_Cu
d_BAC_Cu_CI$yi_lowerCI <- d_BAC_Cu_CI$yi-(1.96*sqrt(d_BAC_Cu_CI$vi))
d_BAC_Cu_CI$yi_upperCI <- d_BAC_Cu_CI$yi+(1.96*sqrt(d_BAC_Cu_CI$vi))
d_BAC_Cu_CI$yiCIsign <- d_BAC_Cu_CI$yi_upperCI*d_BAC_Cu_CI$yi_lowerCI
d_BAC_Cu_CI$LnRR <- ifelse(d_BAC_Cu_CI$yiCIsign<0, "p>=0.05", "p<0.05") #filter only significant values
d_BAC_Cu_CI_signif<-d_BAC_Cu_CI%>%filter(LnRR == "p<0.05") 


#Figure 2 ####
ggplot(d_BAC_Cu_CI, aes(x=yi))+
  geom_histogram(data=d_BAC_Cu_CI, binwidth = 0.05, fill="gray")+
  geom_histogram(data=d_BAC_Cu_CI_signif, binwidth = 0.05, fill="darkgreen")+
  theme_bw()+ ylim(0,60)+
  geom_vline(xintercept = -0.1032, color = "blue", size=0.75)+
  geom_vline(xintercept = -0.1032-0.0340, color = "blue", size=0.5, linetype = "3313")+
  geom_vline(xintercept = -0.1032+0.0340, color = "blue", size=0.5, linetype = "3313")+ 
  xlab("lnRR microbial alpha-diversity")+
  theme(axis.text = element_text(color = "black", size = 12)) +
  theme(axis.title.x = element_text(color = "black", size = 12))+
  theme(axis.title.y = element_text(color = "black", size = 12)) 



# MODEL SELECTIONS Cd-Cu ----------------------------------------------------------------------------------
#set the random component
rand = list(~ 1 | study/experiment/id, ~ time_hours_days | experiment)

# Cu ______________________________________________________________________________________________________________________________________
mods<-c("bio_system","addition_mg_kg","amended","soilpH","soilOM","diversity_metric")
model_selection_BAC_Cu<-lowest_BIC_selection(data=d_BAC_Cu, mods = mods, rand = rand)
print_model_selection_output(model_selection_BAC_Cu,
                             lowest_BIC = T)

model_selection_BAC_Cu_data<-as.data.frame(print_model_selection_output(model_selection_BAC_Cu,
                                                                        lowest_BIC = T))
write.csv(model_selection_BAC_Cu_data, file="model selection Cu")

top_model_bac_Cu<-get_REML_mod(model = model_selection_BAC_Cu[length(model_selection_BAC_Cu)][[1]], # "x[length(x)][[1]]" last element of the x list, i.e. is the top model
                               data=d_BAC_Cu)
top_model_bac_Cu

model_BAC_Cu_dose_pH<-rma.mv(yi~addition_mg_kg*soilpH, get_V(d_BAC_Cu), data=d_BAC_Cu, random = rand, struct = "CAR")
# no interaction effect found. 



# Cd _________________________________________________________________________________________________________________________________
mods<-c("bio_system","addition_mg_kg","amended","soilpH","soilOM","diversity_metric")
model_selection_BAC_Cd<-lowest_BIC_selection(data=d_BAC_Cd, mods = mods, rand = rand)
print_model_selection_output(model_selection_BAC_Cd,
                             lowest_BIC = T)#"Formula top model: yi ~ NA"
#For Cd, no model other than the null one is the best

write.csv(model_selection_BAC_Cd_data, file="model selection Cd")


fake_top_model_bac_Cd <-rma.mv(yi~addition_mg_kg+soilpH, get_V(d_BAC_Cd), data=d_BAC_Cd, random = rand, struct = "CAR")
fake_top_model_bac_Cd #For Cd, get a model similar to the best model for Cu subset. You are going to need it to plot. 




#Figure 3 ####
# metal additions_______________________________________________________________________________________________________________
pred.to.model_Cd<- seq( d_BAC_Cd |> pull(addition_mg_kg) |> min(), 
                        d_BAC_Cd |>  pull(addition_mg_kg) |> max(), 
                        length=208)


pred.to.model_Cu<- seq( d_BAC_Cu |> pull(addition_mg_kg) |> min(),
                        d_BAC_Cu |>  pull(addition_mg_kg) |> max(),
                        length=245)

head(cbind(pred.to.model_Cu, 0)) 

pred_Cd <- predict(fake_top_model_bac_Cd, newmods=cbind(pred.to.model_Cd, 0)) %>% as.data.frame()
pred_Cu <- predict(top_model_bac_Cu, newmods=cbind(pred.to.model_Cu, 0)) %>% as.data.frame() 

pred_Cd$metal <- 'Cd' 
pred_Cu$metal <- 'Cu'

pred_Cd$pred.to.model <- pred.to.model_Cd 
pred_Cu$pred.to.model <- pred.to.model_Cu

#Cu
ggplot() +  
  geom_hline(yintercept = 0, col="black", size=0.75)+
  geom_point(data = d_BAC_Cu, aes(x=addition_mg_kg, y=yi), fill="grey50", col="grey50", size=7, alpha=.3) +
  geom_ribbon(data = pred_Cu, aes(x=pred.to.model, ymin=ci.lb, ymax=ci.ub, fill=metal), alpha=.5, linetype=2, fill="darkgreen", col="black")+
  geom_line(data=pred_Cu, aes(x=pred.to.model, y=pred), col="darkgreen", size=1.5) +
  theme_bw()+ geom_vline(xintercept = -0.1275, col="black", size=1.3)+
  theme(axis.text = element_text(color = "black", size = 12)) +
  theme(axis.title.x = element_text(color = "black", size = 12))+
  theme(axis.title.y = element_text(color = "black", size = 12))+ 
  scale_y_continuous(limits = c(-1.75, 1), breaks = c(-1.5, -1, -0.5, 0, 0.5), labels=c("-78", "-63", "-39", "0", "64"))+ 
  scale_x_continuous(limits = c(-1.689996, 2.115276), breaks = c(-1.689996, -1, 0, 1, 2.115276), labels=c("1", "4", "39", "342", "3837"))+
  xlab("Metal dose (mg kg-1)")+ 
  ylab("Alpha-diversity change (%)") #OK!!!!!!

#Cd
ggplot() +  
  geom_hline(yintercept = 0, col="black", size=0.75)+
  geom_point(data = d_BAC_Cd, aes(x=addition_mg_kg, y=yi), fill="grey50", col="grey50", size=7, alpha=.3) +
  geom_ribbon(data = pred_Cd, aes(x=pred.to.model, ymin=ci.lb, ymax=ci.ub, fill=metal), alpha=.5, linetype=2, fill="violet", col="black")+
  geom_line(data=pred_Cd, aes(x=pred.to.model, y=pred), col="violet", size=1.5) +
  theme_bw()+
  theme(axis.text = element_text(color = "black", size = 12)) +
  theme(axis.title.x = element_text(color = "black", size = 12))+
  theme(axis.title.y = element_text(color = "black", size = 12))+ 
  scale_y_continuous(limits = c(-0.5, 0.6), breaks = c(-0.5, 0, 0.5), labels=c("-39", "0", "64"))+ 
  scale_x_continuous(limits = c(-2.246, 1.76), breaks = c(-2.245157, -1, 0, 1, 1.758409), labels=c("0.3", "4", "39", "342", "1770"))+
  xlab("Metal dose (mg kg-1)")+ 
  ylab("Alpha-diversity change (%)") #OK!!!!!!


# matrix pH_______________________________________________________________________________________________________________
pred.to.model_Cd<- seq( d_BAC_Cd |> pull(soilpH) |> min(), #minimo valore di x
                        d_BAC_Cd |>  pull(soilpH) |> max(), #massimo valore di x 
                        length=208)


pred.to.model_Cu<- seq( d_BAC_Cu |> pull(soilpH) |> min(), #minimo valore di x
                        d_BAC_Cu |>  pull(soilpH) |> max(), #massimo valore di x 
                        length=245)# in newmods mettiamo dentro una matrice che tenga fisso a 0 i valori che non ci interessa predirre, mentre fornimo dei valori variabili dove vogliamo predirre

head(cbind(pred.to.model_Cu, 0)) 

#... quindi:
pred_Cd <- predict(fake_top_model_bac_Cd, newmods=cbind(1, pred.to.model_Cd)) %>% as.data.frame()
pred_Cu <- predict(top_model_bac_Cu, newmods=cbind(1, pred.to.model_Cu)) %>% as.data.frame() # trasformo tutto in dataframe per poter plottare meglio dopo

#plot dell'interazione con ggplot
pred_Cd$metal <- 'Cd' # assegniamo i nomi a ciascun dataframe con le predizioni (pred e' il valore predetto sull'asse delle y!)
pred_Cu$metal <- 'Cu'

pred_Cd$pred.to.model <- pred.to.model_Cd # riaggiungiamo il valore di x per plottare le linee in geom_line
pred_Cu$pred.to.model <- pred.to.model_Cu



#soglia di decremento in base a pH
predict(top_model_bac_Cu,newmods=c(1, 1.0231473864), addx = TRUE) #a -0.0275 di aggiunta rame la a-diversity comincia a calare
1.0231473864 * attr(d$soilpH, 'scaled:scale')+attr(d$soilpH, 'scaled:center')



#Cu
ggplot() +  
  geom_hline(yintercept = 0, col="black", size=0.75)+
  geom_point(data = d_BAC_Cu, aes(x=soilpH, y=yi), fill="grey50", col="grey50", size=7, alpha=.3) +
  geom_ribbon(data = pred_Cu, aes(x=pred.to.model, ymin=ci.lb, ymax=ci.ub, fill=metal), alpha=.5, linetype=2, fill="darkgreen", col="black")+
  geom_line(data=pred_Cu, aes(x=pred.to.model, y=pred), col="darkgreen", size=1.5) +
  theme_bw()+ 
  theme(axis.text = element_text(color = "black", size = 12)) +
  theme(axis.title.x = element_text(color = "black", size = 12))+
  theme(axis.title.y = element_text(color = "black", size = 12))+   
  xlab("pH")+ 
  ylab("Alpha-diversity change (%)")+
  scale_y_continuous(limits = c(-1.70, 0.8), breaks = c(-1.5, -1, -0.5, 0, 0.5), labels=c("-78", "-63", "-39", "0", "64"))+ 
  scale_x_continuous(limits = c(-1.78, 1.1), breaks = c(-1.78, -1, 0, 1.1), labels=c("4", "5", "6.6", "8")) #OK!!!!!!

#Cd
ggplot() +  
  geom_hline(yintercept = 0, col="black", size=0.75)+
  geom_point(data = d_BAC_Cd, aes(x=soilpH, y=yi), fill="grey50", col="grey50", size=7, alpha=.3)+
  geom_ribbon(data = pred_Cd, aes(x=pred.to.model, ymin=ci.lb, ymax=ci.ub, fill=metal), alpha=.5, linetype=2, fill="violet", col="black")+
  geom_line(data=pred_Cd, aes(x=pred.to.model, y=pred), col="violet", size=1.5) +
  theme_bw()+ 
  theme(axis.text = element_text(color = "black", size = 12)) +
  theme(axis.title.x = element_text(color = "black", size = 12))+
  theme(axis.title.y = element_text(color = "black", size = 12))+   
  xlab("matrix pH")+ 
  ylab("Alpha-diversity change (%)")+
  scale_y_continuous(limits = c(-0.4, 0.55), breaks = c(-0.3929077, 0, 0.5392952), labels=c("-32", "0", "71"))+ 
  scale_x_continuous(limits = c(-2, 1.65), breaks = c(-1.988295, -1, 0, 1.637621), labels=c("4", "5", "6.6", "8.8")) #OK!!!!!! #OK!!!!!!




#diversity metrics -------------------------------------------------------------------------------------------------
rand = list(~ 1 | study/experiment/id, ~ time_hours_days | experiment)

X_BAC_Cu<-rma.mv(yi ~ diversity_metric, vi, data=d_BAC_Cu, rand = rand, method="REML", struct = "CAR")
X_BAC_Cu
summary_diversity_Cu<-as.data.frame(tidy(X_BAC_Cu))

X_BAC_Cd<-rma.mv(yi ~ diversity_metric, vi, data=d_BAC_Cd, rand = rand, method="REML", struct = "CAR")
X_BAC_Cd
summary_diversity_Cd<-as.data.frame(tidy(X_BAC_Cd))

summary_diversity_Cd$CI.min<-summary_diversity_Cd$estimate-summary_diversity_Cd$std.error
summary_diversity_Cd$CI.max<-summary_diversity_Cd$estimate+summary_diversity_Cd$std.error
summary_diversity_Cd$term<-c("ACE", "Chao1", "Richness", "Shannon-Wiener", "Simpson")
colnames(summary_diversity_Cd)<-c("diversity_metric","type","mean","std.error","statistic","p.value","CI.min","CI.max")
summary_diversity_Cd$diversity_metric<-as.character(summary_diversity_Cd$diversity_metric)

summary_diversity_Cu$CI.min<-summary_diversity_Cu$estimate-summary_diversity_Cu$std.error
summary_diversity_Cu$CI.max<-summary_diversity_Cu$estimate+summary_diversity_Cu$std.error
summary_diversity_Cu$term<-c("ACE", "Chao1", "Richness", "Shannon-Wiener", "Simpson")
colnames(summary_diversity_Cu)<-c("diversity_metric","type","mean","std.error","statistic","p.value","CI.min","CI.max")
summary_diversity_Cu$diversity_metric<-as.character(summary_diversity_Cu$diversity_metric)

# Figure 4 ####
#Cu
ggplot(d_BAC_Cu, aes(diversity_metric,yi)) + theme_bw()+ 
  geom_hline(yintercept=0, col="black", linetype="dashed", size=0.75) +
  geom_boxplot(aes(fill=diversity_metric), alpha=0.9)+ scale_fill_manual(values = c("#046211","#2CC90C","#10704E","#38D185","#0B8A2F"))+
  labs(y="LnRR microbial alpha-diversity variation", x="", fill = "Diversity metric")+ 
  geom_errorbar(data= summary_diversity_Cu, mapping=aes(x= diversity_metric, y=mean, ymax=CI.max, 
                                                        ymin=CI.min), width = 0.2, size=0.7)+
  geom_point(data = summary_diversity_Cu, aes(x=diversity_metric, y=mean), fill="red", shape=22, size=3) + 
  ggtitle("Variation of microbial alpha-diversity indices")+
  theme(plot.title = element_text(size = 12, face = "bold"))+  
  theme(axis.text = element_text(color = "black", size = 12)) +
  theme(axis.title.y = element_text(color = "black", size = 12))

#Cd
ggplot(d_BAC_Cd, aes(diversity_metric,yi)) + theme_bw()+ 
  geom_hline(yintercept=0, col="black", linetype="dashed", size=0.75) +
  geom_boxplot(aes(fill=diversity_metric), alpha=0.9)+ scale_fill_manual(values = c("#E229F0","#B50CF7","#B50CF7","#F68EF1","#BC7DF2"))+
  labs(y="LnRR microbial alpha-diversity variation", x="", fill = "Diversity metric")+ 
  geom_errorbar(data= summary_diversity_Cd, mapping=aes(x= diversity_metric, y=mean, ymax=CI.max, 
                                                        ymin=CI.min), width = 0.2, size=0.7)+
  geom_point(data = summary_diversity_Cd, aes(x=diversity_metric, y=mean), fill="red", shape=22, size=3) + 
  ggtitle("Variation of microbial alpha-diversity indices")+
  theme(plot.title = element_text(size = 12, face = "bold"))+  
  theme(axis.text = element_text(color = "black", size = 12)) +
  theme(axis.title.y = element_text(color = "black", size = 12))




