#R code (functions) for the meta-analysis####

#'The response of bacterial and fungal alpha-diversity to heavy metals in soil, sediment, and rhizosphere: A meta-analysis'
#Authors of the R script: Signorini M. & Midolo G.

#functions for running the meta-analysis

#Load packages
load.pkgs<-function() {
  if (!require("dplyr")) install.packages("dplyr")
  if (!require("ggplot2")) install.packages("ggplot2")
  if (!require("metafor")) install.packages("metafor")
  if (!require("readxl")) install.packages("readxl")
  if (!require("tidyr")) install.packages("tidyr")
  if (!require("matrixcalc")) install.packages("matrixcalc")
  if (!require("stringr")) install.packages("stringr")
  if (!require("tibble")) install.packages("tibble")
  if (!require("stringdist")) install.packages("stringdist")
  if (!require("broom")) install.packages("broom")
  

  library(dplyr)
  library(ggplot2)
  library(metafor)
  library(readxl)
  library(matrixcalc)
  library(tidyr)
  library(stringr)
  library(tibble)
  library(stringdist)  
  library(broom)
}

arrange_meta_table<-function(pth,meta_table_name){
  
  #data cleaning for sheet 4&5 Meta_Table.xlxs file!
  sheet5<-readxl::read_excel(paste(pth,meta_table_name,sep="/"),sheet=5)%>%
    filter(!is.na(study))#removeNA
  
  sheet7<-readxl::read_excel(paste(pth,meta_table_name,sep="/"),sheet=7)
  sheet7$control_sd<-NA #setting newly extracted SD to NA to aviod incongruence when using 'check_se_sd_equivalence'
  
  sheet4<-readxl::read_excel(paste(pth,meta_table_name,sep="/"),sheet=4)%>%
    filter(!is.na(study))%>% #removeNAstudy
    mutate_at(c(6:11,13:20),as.numeric) #make numerics columns
  for (i in 1:nrow(sheet7)) {
    
    Num.i<-sheet7$Num.[i]
    sheet4$control_m[which(sheet4$Num.==sheet7$Num.[i])]<-sheet7$control_m[i]
    sheet4$control_se[which(sheet4$Num.==sheet7$Num.[i])]<-sheet7$control_se[i]
    sheet4$control_sd[which(sheet4$Num.==sheet7$Num.[i])]<-sheet7$control_sd[i]
    
  }
  sheet4<-sheet4%>%filter(!is.na(treatment_m))%>%#rimuoviamo osservazioni mancanti di treatment vs. controllo
    select(-Num.)
  sheet4$id<-1:nrow(sheet4)
  
  #rename stuff, rimuovi spazi e parentesi dai nomi:
  names(sheet5)[9]<-"addition_type"
  names(sheet4)<- c("study","study_id","experiment",
                    "common_observation_id","time_hours_days","soilpH","soilOM",
                    "conc_control_mg_kg",
                    "addition_mg_L","addition_mg_kg","diversity_metric",
                    "control_m","treatment_m","control_se", 
                    "treatment_se","control_n","treatment_n",
                    "control_sd","treatment_sd","id")
  
  #crate final dataset and remove useless data
  dat<-right_join(sheet5,sheet4)%>%filter(experiment!="28_3")%>%filter(experiment!="28_2")%>%filter(experiment!="36_1")
  
  #update del 09/02/21
  #contiamo le observations con SE = 0 come SE = NA sul principio che lo studio non le riporta (cifre significative insuff.)
  dat$control_se<-ifelse(dat$control_se==0 & dat$treatment_se==0, NA, dat$control_se)
  dat$treatment_se<-ifelse(dat$control_se==0 & dat$treatment_se==0, NA, dat$treatment_se)
  dat$control_sd<-ifelse(dat$control_sd==0 & dat$treatment_sd==0, NA, dat$control_sd)
  dat$treatment_sd<-ifelse(dat$control_sd==0 & dat$treatment_sd==0, NA, dat$treatment_sd)
  
  #correzione del common obs id:
  dat<-dat%>%group_by(study,experiment,common_observation_id,control_m)%>%
    mutate(common_observation_id=group_indices())%>%
    rename(common_id=common_observation_id)
  
  #correggiamo i nomi come soil1plant1 eccetera, vengono convertiti in "soilplant"
  dat$bio_system<-ifelse(startsWith(dat$bio_system, "soil"),"soilplant",dat$bio_system)
  
  return(as.data.frame(dat))
}

check_se_sd_equivalence<-function(x){
  xc<-x%>%dplyr::select(id,control_se,control_n,control_sd)
  xc<-xc[complete.cases(xc), ]
  xc$se_sd_diff<-round(xc$control_se,4)-round( (xc$control_sd/ sqrt(xc$control_n)) , 4)
  
  
  xt<-x%>%dplyr::select(id,treatment_se,treatment_n,treatment_sd)
  xt<-xt[complete.cases(xt), ]
  xt$se_sd_diff<-round(xt$treatment_se,4)-round( (xt$treatment_sd/ sqrt(xt$treatment_n)) , 4)
  
  xc$type<-"control"
  xt$type<-"treatment"
  xi<-bind_rows(xt,xc)
  
  xi<-xi%>%filter(se_sd_diff!=0)%>%select(id)
  
  if ( nrow(xi)==0) { print(paste(sep="","ok: in tutti i casi SE = SD/sqrt(n). Numero di casi completi per treatment e control: ",
                                  as.character(nrow(xt)), " e ", as.character(nrow(xc)), " su ", as.character(nrow(x))))
  } else print(xi)
  
}

complete_cases_se.sd.n<-function(x,
                                 control_se="control_se",control_sd="control_sd",control_n="control_n",
                                 treatment_se="treatment_se",treatment_sd="treatment_sd",treatment_n="treatment_n"
){
  xi1<-x
  
  colnames.id<-c(control_se,control_n,control_sd)
  
  xc<-xi1[!complete.cases(xi1[ , c(which(names(xi1)==colnames.id[1]),
                                   which(names(xi1)==colnames.id[2]),
                                   which(names(xi1)==colnames.id[3]))]), ]
  xcf<-xi1%>%anti_join(xc)
  
  xc$control_sd<-ifelse(is.na(xc$control_sd),xc$control_se*sqrt(xc$control_n),xc$control_sd)
  xc$control_se<-ifelse(is.na(xc$control_se),xc$control_sd/sqrt(xc$control_n),xc$control_se)
  xc$control_n<-ifelse(is.na(xc$control_n),(xc$control_sd/xc$control_se)^2,xc$control_n)
  
  xi2<-bind_rows(xcf,xc)
  
  colnames.id<-c(treatment_se,treatment_n,treatment_sd)
  
  xt<-xi2[!complete.cases(xi2[ , c(which(names(xi2)==colnames.id[1]),
                                   which(names(xi2)==colnames.id[2]),
                                   which(names(xi2)==colnames.id[3]))]), ]
  xtf<-xi2%>%anti_join(xt)
  
  xt$treatment_sd<-ifelse(is.na(xt$treatment_sd),xt$treatment_se*sqrt(xt$treatment_n),xt$treatment_sd)
  xt$treatment_se<-ifelse(is.na(xt$treatment_se),xt$treatment_sd/sqrt(xt$treatment_n),xt$treatment_se)
  xt$treatment_n<-ifelse(is.na(xt$treatment_n),(xt$treatment_sd/xt$treatment_se)^2,xt$treatment_n)
  
  xi3<-bind_rows(xtf,xt)%>%arrange(id)%>%as.data.frame()
  return(xi3)
  
}

missing_cases_se.sd.n<-function(x, control_se="control_se",control_sd="control_sd",control_n="control_n",
                                treatment_se="treatment_se",treatment_sd="treatment_sd",treatment_n="treatment_n")
{
  colnames.id<-c(control_se,control_n,control_sd,treatment_se,treatment_n,treatment_sd)
  
  xi_missing<-x[!complete.cases(x[ , c(which(names(x)==colnames.id[1]),
                                       which(names(x)==colnames.id[2]),
                                       which(names(x)==colnames.id[3]),
                                       which(names(x)==colnames.id[4]),
                                       which(names(x)==colnames.id[5]),
                                       which(names(x)==colnames.id[6]))
  ]), ]
  
  xi_complete<-x%>%anti_join(xi_missing)
  
  xls<-list(xi_missing,xi_complete)
  names(xls)<- c("missing_complete_cases","full_complete_cases")
  
  return(xls)#ottieni una lista con i due datasets 
}

impute_missing_sd.n<-function(x)
{
  n_median_treatment<-median(x$treatment_n,na.rm=T)
  n_median_control<-median(x$control_n,na.rm=T)
  # put median for missing n cases:
  x$treatment_n<-ifelse(is.na(x$treatment_n),n_median_treatment,x$treatment_n) 
  x$control_n<-ifelse(is.na(x$control_n),n_median_control,x$control_n)
  
  x<-complete_cases_se.sd.n(x) #ora che abbiamo imputato la n, andiamoci a completare la sd per quei casi in cui abbiamo se (o sd) e la n imputata
  #rimaniamo solo piu con le osservazioni che hanno n ma non hanno nè sd nè se!
  #per queste imputiamo la sd usando l'approccio del coefficiente di variazione
  #Hoeksema et al. (2018) approach (https://doi.org/10.1038/s42003-018-0120-9):
  
  median_t_cv<-x%>%summarize(treatment_cv=treatment_sd/treatment_m)%>%summarize(medianvalue=median(treatment_cv,na.rm=T))
  median_t_cv<-median_t_cv$medianvalue[1]
  
  median_c_cv<-x%>%summarize(control_cv=control_sd/control_m)%>%summarize(medianvalue=median(control_cv,na.rm=T))
  median_c_cv<-median_c_cv$medianvalue[1]
  
  #let's multiply median of CV values to impute missing sd
  x$treatment_sd<-ifelse(is.na(x$treatment_sd),median_t_cv*x$treatment_m,x$treatment_sd) 
  x$control_sd<-ifelse(is.na(x$control_sd),median_c_cv*x$control_m,x$control_sd)
  
  #infine, utilizziamo ancora una volta la complete_cases_se.sd.n() function per calcolare i rimanenti missing SE
  x<-complete_cases_se.sd.n(x)
  #fatto!
  
  return(x)#
}

get_perc_change<-function(x){
  return(round(100*(exp(x)-1),2))
}

get_estimate.and.CI<-function(x){#dove x è un modello di metafor!
  return(
    get_perc_change(c(coef(summary(x))$estimate,coef(summary(x))$ci.lb,coef(summary(x))$ci.ub))
  )
}

largetosmall_var_ratio<-function(x,vi_name,na.rm)
{
  return(max(range(dat[,which(names(x)==vi_name)],na.rm=na.rm))/
           min(range(dat[,which(names(x)==vi_name)],na.rm=na.rm)))
}

calc_bias_corrected_lnrr<-function(x)
{
  
  lnrr_r_t <- (x$treatment_sd^2) / (x$treatment_n*(x$treatment_m^2))
  lnrr_r_c <- (x$control_sd^2) / (x$control_n*(x$control_m^2))
  
  x$dyi <- (x$yi) + 0.5*(lnrr_r_t-lnrr_r_c)
  
  var_r_t <- (x$treatment_sd^4) / ((x$treatment_n^2)*(x$treatment_m^4))
  var_r_c <- (x$control_sd^4) / ((x$control_n^2)*(x$control_m^4))
  
  x$dvi <- (x$vi) + 0.5*(var_r_t+var_r_c)
  
  return(x)
}

calc.v <- function(x) {
  v <- matrix((x$control_sd[1]^2 / (x$control_n[1] * x$control_m[1]^2)) , nrow=nrow(x), ncol=nrow(x))
  diag(v) <- x$vi
  v
} 

isPosDef <- function(x) all(eigen(x)$values > 0)

get_V<-function(x) {
  return(bldiag(lapply(split(x,x$common_id), calc.v)))
} 

source2 <- function(file, start, end, ...) {
  file.lines <- scan(file, what=character(), skip=start-1, nlines=end-start+1, sep='\n')
  file.lines.collapsed <- paste(file.lines, collapse='\n')
  source(textConnection(file.lines.collapsed), ...)
}

reset_csv_file<-function(){
  source2(paste(scrpt_pth,"scr2.R",sep="/"),start = 1, end=100)
} 

check_NA_values<-function(x){
  
  ycount<-x%>%select(-conc_control_mg_kg,-addition_mg_L)
  ycount<-ycount[rowSums(is.na(ycount)) > 0, ]
  y<-ycount%>%
    gather("missing_value","val",3:ncol(.))%>%filter(is.na(val))%>%
    select(-val)%>%unique()
  
  print(paste("Osservazioni mancanti (NA) in totale: ",nrow(ycount)))
  return(y)
}

set_regression_data<-function(data, remove.na){
  
  d1<-data%>%select(-conc_control_mg_kg,-addition_mg_L)
  
  if (isTRUE(remove.na)) { 
    
    nad <- d1[rowSums(is.na(d1))>0, ]
    d2 <- d1%>%anti_join(nad)
    d2$soilpH<-scale(d2$soilpH)
    d2$soilOM<-scale(log(d2$soilOM))
    d2$time_hours_days<-scale(log(d2$time_hours_days+1))
    d2$addition_mg_kg<-scale(log(d2$addition_mg_kg))
    return(d2)
    
  } else 
    d2 <- data%>%select(-conc_control_mg_kg,-addition_mg_L)
  d2$soilpH<-scale(d2$soilpH)
  d2$soilOM<-scale(log(d2$soilOM))
  d2$time_hours_days<-scale(log(d2$time_hours_days+1))
  d2$addition_mg_kg<-scale(log(d2$addition_mg_kg))
  return(d2)
  
}

run_model<-function(data,random,moderators,full_model,drop_term){
  
  fixed.effect.formula <- as.formula(paste("yi ~ ",(paste(moderators, collapse = ' + ')),sep=""))
  random.effect.formula <- random
  
  if (isTRUE(full_model)) {
    
    m<-metafor::rma.mv(fixed.effect.formula,get_V(data), data=data, random = random.effect.formula, method = 'ML')
    return(m) } else
      
      fixed.effect.formula <- as.formula(paste("yi ~ ",(paste(moderators[!moderators==drop_term], collapse = ' + ')),sep=""))
  
  m<-metafor::rma.mv(fixed.effect.formula,
                     get_V(data), data=data, random = random.effect.formula, method = 'ML')
  return(m)
  
}

suggest_drop_old<-function(model,moderators){
  
  dfc<-as.data.frame(coef(summary(model)))%>%rownames_to_column("term")%>%
    filter(term!="intrcpt")%>%
    arrange(desc(pval))
  
  ls<-list()
  for (i in 1:nrow(dfc)) {
    #in case first term selected is an interaction ok!
    if (isTRUE(str_detect(dfc$term[i],":"))){
      ok<-TRUE
    } else {
      searchstri<-str_starts(dfc$term[i],moderators, negate = FALSE)
      dfi<-as.data.frame(cbind(moderators,searchstri))
      mod_to_excludei<-as.character(dfi$moderators[which(searchstri==T)])
      dfc_check<-dfc%>%
        filter(grepl(mod_to_excludei,term))
      length_check<-length(unique(str_detect(dfc_check$term,":")))
      if (length_check==1) {
        ok<-TRUE
        
      } else {ok<-FALSE}
    }
    ls[[i]]<-as.data.frame(cbind(ok))
  }
  lsd<-bind_rows(ls)%>%rownames_to_column("ID")%>%filter(ok==TRUE)
  okval<-as.numeric(lsd[1,1])
  
  rawterm<-dfc$term[okval]
  
  if (str_detect(rawterm,":")==TRUE)  {
    val1<-strsplit(rawterm,":")[[1]][1]
    val2<-strsplit(rawterm,":")[[1]][2]
    i1<-1
    chmtch1<-NA
    while(is.na(chmtch1)){
      chmtch1 <- charmatch(str_sub(val1, 1, -i1),moderators)
      i1 = i1+1
    }
    i2<-1
    chmtch2<-NA
    while(is.na(chmtch2)){
      chmtch2 <- charmatch(str_sub(val2, 1, -i2),moderators)
      i2 = i2+1
    }
    dfmatch<-data.frame(cbind(moderators,grepl(moderators[chmtch1],moderators),grepl(moderators[chmtch2],moderators)))%>%filter(V2==T&V3==T)
    mod_to_exclude<-dfmatch$moderators
    return(mod_to_exclude)
    
  } else {
    searchstr<-str_starts(rawterm,moderators, negate = FALSE)
    df<-as.data.frame(cbind(moderators,searchstr))
    mod_to_exclude<-as.character(df$moderators[which(searchstr==T)])
    return(mod_to_exclude)
  }
  
}

suggest_drop<-function(model,moderators){
  
  dfc<-as.data.frame(coef(summary(model)))%>%rownames_to_column("term")%>%
    filter(term!="intrcpt")
  j<-1
  while (j>0) {
    
    dfc<-dfc%>%arrange(desc(pval))
    
    ls<-list()
    for (i in 1:nrow(dfc)) {
      #in case first term selected is an interaction ok!
      if (isTRUE(str_detect(dfc$term[i],":"))){
        ok<-TRUE
      } else {
        searchstri<-str_starts(dfc$term[i],moderators, negate = FALSE)
        dfi<-as.data.frame(cbind(moderators,searchstri))
        mod_to_excludei<-as.character(dfi$moderators[which(searchstri==T)])
        dfc_check<-dfc%>%
          filter(grepl(mod_to_excludei,term))
        length_check<-length(unique(str_detect(dfc_check$term,":")))
        if (length_check==1) {
          ok<-TRUE
          
        } else {ok<-FALSE}
      }
      ls[[i]]<-as.data.frame(cbind(ok))
    }
    lsd<-bind_rows(ls)%>%rownames_to_column("ID")%>%filter(ok==TRUE)
    okval<-as.numeric(lsd[1,1])
    
    rawterm<-dfc$term[okval]
    
    if (str_detect(rawterm,":")==TRUE)  {
      val1<-strsplit(rawterm,":")[[1]][1]
      val2<-strsplit(rawterm,":")[[1]][2]
      i1<-1
      chmtch1<-NA
      while(is.na(chmtch1)){
        chmtch1 <- charmatch(str_sub(val1, 1, -i1),moderators)
        i1 = i1+1
      }
      i2<-1
      chmtch2<-NA
      while(is.na(chmtch2)){
        chmtch2 <- charmatch(str_sub(val2, 1, -i2),moderators)
        i2 = i2+1
      }
      dfmatch<-data.frame(cbind(moderators,grepl(moderators[chmtch1],moderators),grepl(moderators[chmtch2],moderators)))%>%filter(V2==T&V3==T)
      mod_to_exclude<-dfmatch$moderators
      mod_to_exclude
      
    } else {
      searchstr<-str_starts(rawterm,moderators, negate = FALSE)
      df<-as.data.frame(cbind(moderators,searchstr))
      mod_to_exclude<-as.character(df$moderators[which(searchstr==T)])
      mod_to_exclude
    }
    
    #check levels
    if (length(grepl(mod_to_exclude,dfc$term)[which(grepl(mod_to_exclude,dfc$term))])>1) {
      
      if (grepl(":",mod_to_exclude)){
        dfci<-dfc%>%filter(grepl(mod_to_exclude,term))%>%filter(grepl(":",term))
        dfci_anti<-dfc%>%anti_join(dfci,by = c("term", "estimate", "se", "zval", "pval", "ci.lb", "ci.ub"))
        if (min(dfci$pval)<dfci_anti$pval[1] && min(dfci$pval) < 0.05) { #per essere "protetta" l'interazione deve avere entrambi i requisiti: 1) un pval < del secondo candidato; 2) un pval significativo (< 0.05) 
          dfci$pval<-0 #setta a zero per proteggere le interazioni, ovvero per tenerle presenti nel dfc ma non farle piu considerare come candidato per "mod to exclude"
          dfc<-dfci_anti%>%bind_rows(dfci)
        } else {j<-0}
        
        
      } else { 
        dfci<-dfc%>%filter(grepl(mod_to_exclude,term))
        dfci_anti<-dfc%>%anti_join(dfci,by = c("term", "estimate", "se", "zval", "pval", "ci.lb", "ci.ub"))
        if (min(dfci$pval)<dfci_anti$pval[1] && min(dfci$pval) < 0.05) {
          dfc<-dfci_anti
        } else {j<-0}
      }
      
    }else {j<-0}
    
  }
  
  return(mod_to_exclude)
  
}

get_mods<-function(model){
  v<-str_split(as.character(model[["formula.yi"]])[3], " +")
  v<-v[[1]][!v[[1]]=="+"]
  return(c(v))
}

lowest_BIC_selection<-function(data,mods,rand){
  ls<-list()
  
  ICtype<-"BIC"
  fixed.effect.formula1 <- as.formula(paste("yi ~ ",(paste(mods, collapse = ' + ')),sep=""))
  m1<-rma.mv(fixed.effect.formula1, V=get_V(data), random = rand, struct="CAR",data = data,method = 'ML')
  
  ls[["full_model"]]<-m1
  turno<-0
  IClossi<-0
  while (IClossi>=0) {
    
    if (nrow(coef(summary(m1)))==1)
    {IClossi <- -1} else {
      
      if (nrow(coef(summary(m1)))==2)
      {
        print(paste("Sto valutando se droppare ","'",suggest_drop(m1,mods),"' "),sep="")
        m2<-rma.mv(yi, V=get_V(data), random = rand, struct="CAR",data = data,method = 'ML')
      } else {
        print(paste("Sto valutando se droppare ","'",suggest_drop(m1,mods),"' "),sep="")
        fixed.effect.formula2 <- as.formula(paste("yi ~ ",(paste(get_mods(m1)[which(get_mods(m1)!=suggest_drop(m1,get_mods(m1)))], collapse = ' + ')),sep=""))
        m2<-rma.mv(fixed.effect.formula2, V=get_V(data), random = rand,struct="CAR", data = data,method = 'ML')}
      
      anv<-anova(m1,m2)
      ICloss<-anv[["fit.stats.f"]][[ICtype]]-anv[["fit.stats.r"]][[ICtype]]
      LRTpval<-anv$pval
      
      if (ICloss>=0) {print(paste("droppato ","'",suggest_drop(m1,mods),"': ",ICtype, "loss:", -round(ICloss,2), " LRTp-val: ", round(LRTpval,4)),sep="")
      } else {print(paste("stop, mantengo ","'",suggest_drop(m1,mods),"': ",ICtype, "loss:", -round(ICloss,2), " LRTp-val: ", round(LRTpval,4)),sep="")}
      
      IClossi<-ICloss
      
    }
    
    if (IClossi>=0) {
      turno<-turno+1
      ls[[paste("model",turno,sep="_")]]<-m2
      m1<-m2
    } else {
      print(paste("Formula del top model:",
                  as.character(paste("yi ~ ",(paste(get_mods(m1), collapse = ' + ')),sep="")),
                  sep=" ")
      )
    }
    
    
  }
  
  return(ls)
}


print_model_selection_output<- function(model.list,lowest_AIC=FALSE,lowest_BIC=FALSE) {
  ls<-list()
  for (i in 1:length(model.list)) {
    model.name <- names(model.list)[i]
    model.formula <- as.character(paste("yi ~ ",(paste(get_mods(model.list[[i]]), collapse = ' + ')),sep=""))
    BIC<-round(BIC(model.list[[i]]),2)
    AIC<-round(AIC(model.list[[i]]),2)
    ls[[i]]<-data.frame(cbind(model.name,model.formula,BIC,AIC))
  }
  lsd<-bind_rows(ls)
  
  if (lowest_AIC==TRUE) lsd<-lsd%>%arrange(desc(AIC))
  if (lowest_BIC==TRUE) lsd<-lsd%>%arrange(desc(BIC))
  
  return(lsd)
}

get_REML_mod<- function(model,data) { #non usare su null model, non funziona al momento 13/04/21
  fixed.effect.formula <- as.formula(paste("yi ~ ",(paste(get_mods(model), collapse = ' + ')),sep=""))
  rand<-model$random
  m<-rma.mv(fixed.effect.formula, V=get_V(data), random = rand,struct="CAR",data = data,method = 'REML')
  return(m)
}

